"""
URL constants for StandardWeb3.

Provides API and WebSocket URLs for different networks.
"""

api_urls = {
    "Somnia Testnet": "https://story-odyssey-ponder.standardweb3.com",
}
websocket_urls = {
    "Somnia Testnet": "wss://story-odyssey-websocket.standardweb3.com",
}
